library angular2.src.core.application_dart;

import 'package:angular2/di.dart' show Injector, bind, OpaqueToken;
import 'package:angular2/src/facade/lang.dart'
    show
        Type,
        FIELD,
        isBlank,
        isPresent,
        BaseException,
        assertionsEnabled,
        print;
import 'package:angular2/src/facade/dom.dart' show DOM, Element;
import './compiler/compiler.dart' show Compiler, CompilerCache;
import './compiler/view.dart' show ProtoView;
import 'package:angular2/src/reflection/reflection.dart'
    show Reflector, reflector;
import 'package:angular2/change_detection.dart'
    show
        Parser,
        Lexer,
        ChangeDetection,
        dynamicChangeDetection,
        jitChangeDetection;
import './compiler/template_loader.dart' show TemplateLoader;
import './compiler/template_resolver.dart' show TemplateResolver;
import './compiler/directive_metadata_reader.dart' show DirectiveMetadataReader;
import './compiler/directive_metadata.dart' show DirectiveMetadata;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import 'package:angular2/src/facade/async.dart' show Future, PromiseWrapper;
import 'package:angular2/src/core/zone/vm_turn_zone.dart' show VmTurnZone;
import 'package:angular2/src/core/life_cycle/life_cycle.dart' show LifeCycle;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show ShadowDomStrategy, NativeShadowDomStrategy;
import 'package:angular2/src/core/compiler/xhr/xhr.dart' show XHR;
import 'package:angular2/src/core/compiler/xhr/xhr_impl.dart' show XHRImpl;
import 'package:angular2/src/core/events/event_manager.dart' show EventManager;
import 'package:angular2/src/core/events/hammer_gestures.dart'
    show HammerGesturesPlugin;
import 'package:angular2/src/di/binding.dart' show Binding;

Injector _rootInjector;
var _rootBindings = [
  bind(Reflector).toValue(reflector),
  bind(ChangeDetection).toValue(dynamicChangeDetection),
  Compiler,
  CompilerCache,
  TemplateLoader,
  TemplateResolver,
  DirectiveMetadataReader,
  Parser,
  Lexer,
  bind(ShadowDomStrategy).toValue(new NativeShadowDomStrategy()),
  bind(XHR).toValue(new XHRImpl())
];
var appViewToken = new OpaqueToken('AppView');
var appChangeDetectorToken = new OpaqueToken('AppChangeDetector');
var appElementToken = new OpaqueToken('AppElement');
var appComponentAnnotatedTypeToken =
    new OpaqueToken('AppComponentAnnotatedType');
var appDocumentToken = new OpaqueToken('AppDocument');
List<Binding> _injectorBindings(appComponentType) {
  return [
    bind(appDocumentToken).toValue(DOM.defaultDoc()),
    bind(appComponentAnnotatedTypeToken).toFactory((reader) {
      return reader.read(appComponentType);
    }, [DirectiveMetadataReader]),
    bind(appElementToken).toFactory((appComponentAnnotatedType, appDocument) {
      var selector = appComponentAnnotatedType.annotation.selector;
      var element = DOM.querySelector(appDocument, selector);
      if (isBlank(element)) {
        throw new BaseException(
            '''The app selector \"${selector}\" did not match any elements''');
      }
      return element;
    }, [appComponentAnnotatedTypeToken, appDocumentToken]),
    bind(appViewToken).toAsyncFactory((changeDetection, compiler, injector,
        appElement, appComponentAnnotatedType, strategy, eventManager) {
      return compiler.compile(appComponentAnnotatedType.type).then((protoView) {
        var appProtoView = ProtoView.createRootProtoView(protoView, appElement,
            appComponentAnnotatedType,
            changeDetection.createProtoChangeDetector('root'), strategy);
        var view = appProtoView.instantiate(null, eventManager);
        view.hydrate(injector, null, new Object());
        return view;
      });
    }, [
      ChangeDetection,
      Compiler,
      Injector,
      appElementToken,
      appComponentAnnotatedTypeToken,
      ShadowDomStrategy,
      EventManager
    ]),
    bind(appChangeDetectorToken).toFactory((rootView) {
      return rootView.changeDetector;
    }, [appViewToken]),
    bind(appComponentType).toFactory((rootView) {
      return rootView.elementInjectors[0].getComponent();
    }, [appViewToken]),
    bind(LifeCycle).toFactory(() {
      return new LifeCycle(null, assertionsEnabled());
    }, []),
    bind(EventManager).toFactory((zone) {
      var plugins = [new HammerGesturesPlugin()];
      return new EventManager(plugins, zone);
    }, [VmTurnZone])
  ];
}
VmTurnZone _createVmZone(Function givenReporter) {
  var defaultErrorReporter = (exception, stackTrace) {
    var longStackTrace =
        ListWrapper.join(stackTrace, "\n\n-----async gap-----\n");
    print('''${exception}\n\n${longStackTrace}''');
    throw exception;
  };
  var reporter =
      isPresent(givenReporter) ? givenReporter : defaultErrorReporter;
  var zone = new VmTurnZone(enableLongStackTrace: assertionsEnabled());
  zone.initCallbacks(onErrorHandler: reporter);
  return zone;
}
Future bootstrap(Type appComponentType, [List<Binding> bindings = null,
    Function givenBootstrapErrorReporter = null]) {
  var bootstrapProcess = PromiseWrapper.completer();
  var zone = _createVmZone(givenBootstrapErrorReporter);
  zone.run(() {
    var appInjector = _createAppInjector(appComponentType, bindings, zone);
    PromiseWrapper.then(appInjector.asyncGet(appViewToken), (rootView) {
      var lc = appInjector.get(LifeCycle);
      lc.registerWith(zone, rootView.changeDetector);
      lc.tick();
      bootstrapProcess.complete(appInjector);
    }, (err) {
      bootstrapProcess.reject(err);
    });
  });
  return bootstrapProcess.promise;
}
Injector _createAppInjector(
    Type appComponentType, List<Binding> bindings, VmTurnZone zone) {
  if (isBlank(_rootInjector)) _rootInjector = new Injector(_rootBindings);
  var mergedBindings = isPresent(bindings)
      ? ListWrapper.concat(_injectorBindings(appComponentType), bindings)
      : _injectorBindings(appComponentType);
  ListWrapper.push(mergedBindings, bind(VmTurnZone).toValue(zone));
  return _rootInjector.createChild(mergedBindings);
}
