library angular2.src.core.compiler.pipeline.compile_pipeline_dart;

import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import 'package:angular2/src/facade/dom.dart' show Element, Node, DOM;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;
import './compile_step.dart' show CompileStep;
import '../directive_metadata.dart' show DirectiveMetadata;

class CompilePipeline {
  CompileControl _control;
  CompilePipeline(List<CompileStep> steps) {
    this._control = new CompileControl(steps);
  }
  List process(Element rootElement) {
    var results = ListWrapper.create();
    this._process(results, null, new CompileElement(rootElement));
    return results;
  }
  _process(results, CompileElement parent, CompileElement current) {
    var additionalChildren =
        this._control.internalProcess(results, 0, parent, current);
    if (current.compileChildren) {
      var node = DOM.templateAwareRoot(current.element).firstChild;
      while (isPresent(node)) {
        var nextNode = DOM.nextSibling(node);
        if (identical(node.nodeType, Node.ELEMENT_NODE)) {
          this._process(results, current, new CompileElement(node));
        }
        node = nextNode;
      }
    }
    if (isPresent(additionalChildren)) {
      for (var i = 0; i < additionalChildren.length; i++) {
        this._process(results, current, additionalChildren[i]);
      }
    }
  }
}
