library angular2.src.core.compiler.pipeline.default_steps_dart;

import 'package:angular2/change_detection.dart' show ChangeDetection, Parser;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import './property_binding_parser.dart' show PropertyBindingParser;
import './text_interpolation_parser.dart' show TextInterpolationParser;
import './directive_parser.dart' show DirectiveParser;
import './view_splitter.dart' show ViewSplitter;
import './element_binding_marker.dart' show ElementBindingMarker;
import './proto_view_builder.dart' show ProtoViewBuilder;
import './proto_element_injector_builder.dart' show ProtoElementInjectorBuilder;
import './element_binder_builder.dart' show ElementBinderBuilder;
import './shadow_dom_transformer.dart' show ShadowDomTransformer;
import 'package:angular2/src/core/compiler/directive_metadata.dart'
    show DirectiveMetadata;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show ShadowDomStrategy, NativeShadowDomStrategy;
import 'package:angular2/src/facade/lang.dart' show stringify;
import 'package:angular2/src/facade/dom.dart' show DOM;

createDefaultSteps(ChangeDetection changeDetection, Parser parser,
    DirectiveMetadata compiledComponent, List<DirectiveMetadata> directives,
    ShadowDomStrategy shadowDomStrategy) {
  var compilationUnit = stringify(compiledComponent.type);
  var steps = [new ViewSplitter(parser, compilationUnit)];
  if (!(shadowDomStrategy is NativeShadowDomStrategy)) {
    var step = new ShadowDomTransformer(
        compiledComponent, shadowDomStrategy, DOM.defaultDoc().head);
    ListWrapper.push(steps, step);
  }
  steps = ListWrapper.concat(steps, [
    new PropertyBindingParser(parser, compilationUnit),
    new DirectiveParser(directives),
    new TextInterpolationParser(parser, compilationUnit),
    new ElementBindingMarker(),
    new ProtoViewBuilder(changeDetection, shadowDomStrategy),
    new ProtoElementInjectorBuilder(),
    new ElementBinderBuilder(parser, compilationUnit)
  ]);
  return steps;
}
