library angular2.src.core.compiler.pipeline.directive_parser_dart;

import 'package:angular2/src/facade/lang.dart'
    show isPresent, isBlank, BaseException;
import 'package:angular2/src/facade/collection.dart' show List, MapWrapper;
import 'package:angular2/src/facade/dom.dart' show TemplateElement;
import '../selector.dart' show SelectorMatcher;
import '../selector.dart' show CssSelector;
import '../directive_metadata.dart' show DirectiveMetadata;
import '../../annotations/annotations.dart' show Component, Viewport;
import './compile_step.dart' show CompileStep;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;
import '../shadow_dom_strategy.dart' show ShadowDomStrategy;

class DirectiveParser extends CompileStep {
  SelectorMatcher _selectorMatcher;
  DirectiveParser(List<DirectiveMetadata> directives) : super() {
    this._selectorMatcher = new SelectorMatcher();
    for (var i = 0; i < directives.length; i++) {
      var directiveMetadata = directives[i];
      this._selectorMatcher.addSelectable(
          CssSelector.parse(directiveMetadata.annotation.selector),
          directiveMetadata);
    }
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    var attrs = current.attrs();
    var classList = current.classList();
    var cssSelector = new CssSelector();
    cssSelector.setElement(current.element.nodeName);
    for (var i = 0; i < classList.length; i++) {
      cssSelector.addClassName(classList[i]);
    }
    MapWrapper.forEach(attrs, (attrValue, attrName) {
      if (isBlank(current.propertyBindings) ||
          isPresent(current.propertyBindings) &&
              !MapWrapper.contains(current.propertyBindings, attrName)) {
        cssSelector.addAttribute(attrName, attrValue);
      }
    });
    if (isPresent(current.propertyBindings)) {
      MapWrapper.forEach(current.propertyBindings, (expression, prop) {
        cssSelector.addAttribute(prop, expression.source);
      });
    }
    if (isPresent(current.variableBindings)) {
      MapWrapper.forEach(current.variableBindings, (value, name) {
        cssSelector.addAttribute(name, value);
      });
    }
    var isTemplateElement = current.element is TemplateElement;
    this._selectorMatcher.match(cssSelector, (directive) {
      if (directive.annotation is Viewport) {
        if (!isTemplateElement) {
          throw new BaseException(
              'Viewport directives need to be placed on <template> elements or elements with template attribute!');
        } else if (isPresent(current.viewportDirective)) {
          throw new BaseException(
              'Only one template directive per element is allowed!');
        }
      } else if (isTemplateElement) {
        throw new BaseException(
            'Only template directives are allowed on <template> elements!');
      } else if ((directive.annotation is Component) &&
          isPresent(current.componentDirective)) {
        throw new BaseException(
            'Only one component directive per element is allowed!');
      }
      current.addDirective(directive);
    });
  }
}
