library angular2.src.core.compiler.pipeline.shadow_dom_transformer_dart;

import './compile_step.dart' show CompileStep;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;
import 'package:angular2/src/core/compiler/directive_metadata.dart'
    show DirectiveMetadata;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show ShadowDomStrategy;
import 'package:angular2/src/core/compiler/shadow_dom_emulation/shim_css.dart'
    show shimCssText;
import 'package:angular2/src/facade/dom.dart' show DOM, Element;
import 'package:angular2/src/facade/lang.dart' show isPresent, isBlank;
import 'package:angular2/src/facade/collection.dart' show StringMapWrapper;

var _cssCache = StringMapWrapper.create();
class ShadowDomTransformer extends CompileStep {
  String _selector;
  ShadowDomStrategy _strategy;
  Element _styleHost;
  Element _lastInsertedStyle;
  ShadowDomTransformer(DirectiveMetadata cmpMetadata,
      ShadowDomStrategy strategy, Element styleHost)
      : super() {
    this._strategy = strategy;
    this._selector = cmpMetadata.annotation.selector;
    this._styleHost = styleHost;
    this._lastInsertedStyle = null;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    if (DOM.tagName(current.element) == 'STYLE') {
      current.ignoreBindings = true;
      if (this._strategy.extractStyles()) {
        DOM.remove(current.element);
        var css = DOM.getText(current.element);
        if (this._strategy.shim()) {
          css = shimCssText(css, this._selector);
          this._insertStyle(this._styleHost, css);
        } else {
          var seen = isPresent(StringMapWrapper.get(_cssCache, css));
          if (!seen) {
            StringMapWrapper.set(_cssCache, css, true);
            this._insertStyle(this._styleHost, css);
          }
        }
      }
    } else {
      if (this._strategy.shim()) {
        try {
          DOM.setAttribute(current.element, this._selector, '');
        } catch (e) {}
      }
    }
  }
  clearCache() {
    _cssCache = StringMapWrapper.create();
  }
  _insertStyle(Element el, String css) {
    var style = DOM.createStyleElement(css);
    if (isBlank(this._lastInsertedStyle)) {
      var firstChild = DOM.firstChild(el);
      if (isPresent(firstChild)) {
        DOM.insertBefore(firstChild, style);
      } else {
        DOM.appendChild(el, style);
      }
    } else {
      DOM.insertAfter(this._lastInsertedStyle, style);
    }
    this._lastInsertedStyle = style;
  }
}
