library angular2.src.core.compiler.pipeline.view_splitter_dart;

import 'package:angular2/src/facade/lang.dart'
    show isBlank, isPresent, BaseException;
import 'package:angular2/src/facade/dom.dart' show DOM, TemplateElement;
import 'package:angular2/src/facade/collection.dart'
    show MapWrapper, ListWrapper;
import 'package:angular2/change_detection.dart' show Parser;
import './compile_step.dart' show CompileStep;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;
import 'package:angular2/src/facade/lang.dart' show StringWrapper;
import 'package:angular2/src/change_detection/parser/lexer.dart' show $BANG;

class ViewSplitter extends CompileStep {
  Parser _parser;
  dynamic _compilationUnit;
  ViewSplitter(Parser parser, dynamic compilationUnit) : super() {
    this._parser = parser;
    this._compilationUnit = compilationUnit;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    if (isBlank(parent)) {
      current.isViewRoot = true;
    } else {
      if (current.element is TemplateElement) {
        if (!current.isViewRoot) {
          var viewRoot = new CompileElement(DOM.createTemplate(''));
          TemplateElement currentElement = current.element;
          TemplateElement viewRootElement = viewRoot.element;
          this._moveChildNodes(currentElement.content, viewRootElement.content);
          viewRoot.isViewRoot = true;
          control.addChild(viewRoot);
        }
      } else {
        var attrs = current.attrs();
        var templateBindings = MapWrapper.get(attrs, 'template');
        var hasTemplateBinding = isPresent(templateBindings);
        MapWrapper.forEach(attrs, (attrValue, attrName) {
          if (StringWrapper.charCodeAt(attrName, 0) == $BANG) {
            var key = StringWrapper.substring(attrName, 1);
            if (hasTemplateBinding) {
              throw new BaseException(
                  '''Only one template directive per element is allowed: ''' +
                      '''${templateBindings} and ${key} cannot be used simultaneously!''');
            } else {
              templateBindings =
                  (attrValue.length == 0) ? key : key + ' ' + attrValue;
              hasTemplateBinding = true;
            }
          }
        });
        if (hasTemplateBinding) {
          var newParent = new CompileElement(DOM.createTemplate(''));
          current.isViewRoot = true;
          this._parseTemplateBindings(templateBindings, newParent);
          this._addParentElement(current.element, newParent.element);
          control.addParent(newParent);
          current.element.remove();
        }
      }
    }
  }
  _moveChildNodes(source, target) {
    while (isPresent(source.firstChild)) {
      DOM.appendChild(target, source.firstChild);
    }
  }
  _addParentElement(currentElement, newParentElement) {
    DOM.insertBefore(currentElement, newParentElement);
    DOM.appendChild(newParentElement, currentElement);
  }
  _parseTemplateBindings(
      String templateBindings, CompileElement compileElement) {
    var bindings = this._parser.parseTemplateBindings(
        templateBindings, this._compilationUnit);
    for (var i = 0; i < bindings.length; i++) {
      var binding = bindings[i];
      if (binding.keyIsVar) {
        compileElement.addVariableBinding(binding.key, binding.name);
      } else if (isPresent(binding.expression)) {
        compileElement.addPropertyBinding(binding.key, binding.expression);
      } else {
        compileElement.element.setAttribute(binding.key, '');
      }
    }
  }
}
