library angular2.src.core.compiler.shadow_dom_emulation.content_tag_dart;

import '../../annotations/annotations.dart' show Decorator;
import './light_dom.dart' show SourceLightDom, DestinationLightDom, LightDom;
import 'package:angular2/di.dart' show Inject;
import 'package:angular2/src/facade/dom.dart' show Element, Node, DOM;
import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import 'package:angular2/src/core/dom/element.dart' show NgElement;

var _scriptTemplate = DOM.createScriptTag('type', 'ng/content');
class ContentStrategy {
  List<Node> nodes;
  insert(List<Node> nodes) {}
}
class RenderedContent extends ContentStrategy {
  Element beginScript;
  Element endScript;
  RenderedContent(Element contentEl) : super() {
    this._replaceContentElementWithScriptTags(contentEl);
    this.nodes = [];
  }
  insert(List<Node> nodes) {
    this.nodes = nodes;
    DOM.insertAllBefore(this.endScript, nodes);
    this._removeNodesUntil(
        ListWrapper.isEmpty(nodes) ? this.endScript : nodes[0]);
  }
  _replaceContentElementWithScriptTags(Element contentEl) {
    this.beginScript = DOM.clone(_scriptTemplate);
    this.endScript = DOM.clone(_scriptTemplate);
    DOM.insertBefore(contentEl, this.beginScript);
    DOM.insertBefore(contentEl, this.endScript);
    DOM.removeChild(DOM.parentElement(contentEl), contentEl);
  }
  _removeNodesUntil(Node node) {
    var p = DOM.parentElement(this.beginScript);
    for (var next = DOM.nextSibling(this.beginScript);
        !identical(next, node);
        next = DOM.nextSibling(this.beginScript)) {
      DOM.removeChild(p, next);
    }
  }
}
class IntermediateContent extends ContentStrategy {
  LightDom destinationLightDom;
  IntermediateContent(LightDom destinationLightDom) : super() {
    this.destinationLightDom = destinationLightDom;
    this.nodes = [];
  }
  insert(List<Node> nodes) {
    this.nodes = nodes;
    this.destinationLightDom.redistribute();
  }
}
@Decorator(selector: 'content')
class Content {
  String select;
  ContentStrategy _strategy;
  Content(
      @Inject(DestinationLightDom) destinationLightDom, NgElement contentEl) {
    this.select = contentEl.getAttribute('select');
    this._strategy = isPresent(destinationLightDom)
        ? new IntermediateContent(destinationLightDom)
        : new RenderedContent(contentEl.domElement);
  }
  List<Node> nodes() {
    return this._strategy.nodes;
  }
  insert(List<Node> nodes) {
    this._strategy.insert(nodes);
  }
}
