library angular2.src.core.compiler.shadow_dom_emulation.light_dom_dart;

import 'package:angular2/src/facade/dom.dart' show Element, Node, DOM;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import 'package:angular2/src/facade/lang.dart' show isBlank, isPresent;
import '../view.dart' show View;
import '../element_injector.dart' show ElementInjector;
import '../view_container.dart' show ViewContainer;
import './content_tag.dart' show Content;

class SourceLightDom {}
class DestinationLightDom {}
class _Root {
  Node node;
  ElementInjector injector;
  _Root(node, injector) {
    this.node = node;
    this.injector = injector;
  }
}
class LightDom {
  View lightDomView;
  View shadowDomView;
  List<Node> nodes;
  List<_Root> roots;
  LightDom(View lightDomView, View shadowDomView, Element element) {
    this.lightDomView = lightDomView;
    this.shadowDomView = shadowDomView;
    this.nodes = DOM.childNodesAsList(element);
    this.roots = null;
  }
  redistribute() {
    var tags = this.contentTags();
    if (tags.length > 0) {
      redistributeNodes(tags, this.expandedDomNodes());
    }
  }
  List<Content> contentTags() {
    return this._collectAllContentTags(this.shadowDomView, []);
  }
  List<Content> _collectAllContentTags(View view, List<Content> acc) {
    var eis = view.elementInjectors;
    for (var i = 0; i < eis.length; ++i) {
      var ei = eis[i];
      if (isBlank(ei)) continue;
      if (ei.hasDirective(Content)) {
        ListWrapper.push(acc, ei.get(Content));
      } else if (ei.hasPreBuiltObject(ViewContainer)) {
        var vc = ei.get(ViewContainer);
        ListWrapper.forEach(vc.contentTagContainers(), (view) {
          this._collectAllContentTags(view, acc);
        });
      }
    }
    return acc;
  }
  List expandedDomNodes() {
    var res = [];
    var roots = this._roots();
    for (var i = 0; i < roots.length; ++i) {
      var root = roots[i];
      var ei = root.injector;
      if (isPresent(ei) && ei.hasPreBuiltObject(ViewContainer)) {
        var vc = root.injector.get(ViewContainer);
        res = ListWrapper.concat(res, vc.nodes());
      } else if (isPresent(ei) && ei.hasDirective(Content)) {
        var content = root.injector.get(Content);
        res = ListWrapper.concat(res, content.nodes());
      } else {
        ListWrapper.push(res, root.node);
      }
    }
    return res;
  }
  _roots() {
    if (isPresent(this.roots)) return this.roots;
    var viewInj = this.lightDomView.elementInjectors;
    this.roots = ListWrapper.map(this.nodes, (n) {
      return new _Root(n, ListWrapper.find(viewInj, (inj) {
        return inj.forElement(n);
      }));
    });
    return this.roots;
  }
}
redistributeNodes(List<Content> contents, List<Node> nodes) {
  for (var i = 0; i < contents.length; ++i) {
    var content = contents[i];
    var select = content.select;
    var matchSelector = (n) {
      return DOM.elementMatches(n, select);
    };
    if (isBlank(select)) {
      content.insert(nodes);
      ListWrapper.clear(nodes);
    } else {
      var matchingNodes = ListWrapper.filter(nodes, matchSelector);
      content.insert(matchingNodes);
      ListWrapper.removeAll(nodes, matchingNodes);
    }
  }
}
