library angular2.src.core.compiler.shadow_dom_strategy_dart;

import 'package:angular2/src/facade/lang.dart' show Type, isBlank, isPresent;
import 'package:angular2/src/facade/dom.dart' show DOM, Element;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import './view.dart' show View;
import './shadow_dom_emulation/content_tag.dart' show Content;
import './shadow_dom_emulation/light_dom.dart' show LightDom;
import './directive_metadata.dart' show DirectiveMetadata;

class ShadowDomStrategy {
  attachTemplate(Element el, View view) {}
  constructLightDom(View lightDomView, View shadowDomView, Element el) {}
  List<Type> polyfillDirectives() {
    return null;
  }
  bool shim() {
    return false;
  }
  bool extractStyles() {
    return false;
  }
}
class EmulatedShadowDomStrategy extends ShadowDomStrategy {
  EmulatedShadowDomStrategy() : super() {}
  attachTemplate(Element el, View view) {
    DOM.clearNodes(el);
    moveViewNodesIntoParent(el, view);
  }
  constructLightDom(View lightDomView, View shadowDomView, Element el) {
    return new LightDom(lightDomView, shadowDomView, el);
  }
  List<Type> polyfillDirectives() {
    return [Content];
  }
  bool shim() {
    return true;
  }
  bool extractStyles() {
    return true;
  }
}
class NativeShadowDomStrategy extends ShadowDomStrategy {
  NativeShadowDomStrategy() : super() {}
  attachTemplate(Element el, View view) {
    moveViewNodesIntoParent(el.createShadowRoot(), view);
  }
  constructLightDom(View lightDomView, View shadowDomView, Element el) {
    return null;
  }
  List<Type> polyfillDirectives() {
    return [];
  }
  bool shim() {
    return false;
  }
  bool extractStyles() {
    return false;
  }
}
moveViewNodesIntoParent(parent, view) {
  for (var i = 0; i < view.nodes.length; ++i) {
    DOM.appendChild(parent, view.nodes[i]);
  }
}
