library angular2.src.core.compiler.template_loader_dart;

import 'package:angular2/src/facade/async.dart' show Future, PromiseWrapper;
import 'package:angular2/src/facade/lang.dart'
    show isBlank, isPresent, BaseException, stringify;
import 'package:angular2/src/facade/dom.dart' show DOM, Element;
import 'package:angular2/src/facade/collection.dart' show StringMapWrapper;
import './xhr/xhr.dart' show XHR;
import 'package:angular2/src/core/annotations/template.dart' show Template;

class TemplateLoader {
  XHR _xhr;
  var _cache;
  TemplateLoader(XHR xhr) {
    this._xhr = xhr;
    this._cache = StringMapWrapper.create();
  }
  load(Template template) {
    if (isPresent(template.inline)) {
      return DOM.createTemplate(template.inline);
    }
    if (isPresent(template.url)) {
      var url = template.url;
      var promise = StringMapWrapper.get(this._cache, url);
      if (isBlank(promise)) {
        promise = this._xhr.get(url).then((html) {
          var template = DOM.createTemplate(html);
          return template;
        });
        StringMapWrapper.set(this._cache, url, promise);
      }
      return promise;
    }
    throw new BaseException(
        '''Templates should have either their url or inline property set''');
  }
}
