library angular2.src.core.compiler.view_container_dart;

import './view.dart' show View, ProtoView;
import 'package:angular2/src/facade/dom.dart' show DOM, Node, Element;
import 'package:angular2/src/facade/collection.dart'
    show ListWrapper, MapWrapper, List;
import 'package:angular2/src/facade/lang.dart' show BaseException;
import 'package:angular2/di.dart' show Injector;
import 'package:angular2/src/core/compiler/element_injector.dart'
    show ElementInjector;
import 'package:angular2/src/facade/lang.dart' show isPresent, isBlank;
import 'package:angular2/src/core/events/event_manager.dart' show EventManager;

class ViewContainer {
  View parentView;
  Element templateElement;
  ProtoView defaultProtoView;
  List<View> _views;
  dynamic _lightDom;
  EventManager _eventManager;
  ElementInjector elementInjector;
  Injector appInjector;
  ElementInjector hostElementInjector;
  ViewContainer(View parentView, Element templateElement,
      ProtoView defaultProtoView, ElementInjector elementInjector,
      EventManager eventManager, [lightDom = null]) {
    this.parentView = parentView;
    this.templateElement = templateElement;
    this.defaultProtoView = defaultProtoView;
    this.elementInjector = elementInjector;
    this._lightDom = lightDom;
    this._views = [];
    this.appInjector = null;
    this.hostElementInjector = null;
    this._eventManager = eventManager;
  }
  hydrate(Injector appInjector, ElementInjector hostElementInjector) {
    this.appInjector = appInjector;
    this.hostElementInjector = hostElementInjector;
  }
  dehydrate() {
    this.appInjector = null;
    this.hostElementInjector = null;
    this.clear();
  }
  clear() {
    for (var i = this._views.length - 1; i >= 0; i--) {
      this.remove(i);
    }
  }
  View get(num index) {
    return this._views[index];
  }
  get length {
    return this._views.length;
  }
  _siblingToInsertAfter(num index) {
    if (index == 0) return this.templateElement;
    return ListWrapper.last(this._views[index - 1].nodes);
  }
  hydrated() {
    return isPresent(this.appInjector);
  }
  View create([atIndex = -1]) {
    if (!this.hydrated()) throw new BaseException(
        'Cannot create views on a dehydrated ViewContainer');
    var newView = this.defaultProtoView.instantiate(
        this.hostElementInjector, this._eventManager);
    newView.hydrate(
        this.appInjector, this.hostElementInjector, this.parentView.context);
    return this.insert(newView, atIndex);
  }
  View insert(view, [atIndex = -1]) {
    if (atIndex == -1) atIndex = this._views.length;
    ListWrapper.insert(this._views, atIndex, view);
    if (isBlank(this._lightDom)) {
      ViewContainer.moveViewNodesAfterSibling(
          this._siblingToInsertAfter(atIndex), view);
    } else {
      this._lightDom.redistribute();
    }
    this.parentView.changeDetector.addChild(view.changeDetector);
    this._linkElementInjectors(view);
    return view;
  }
  remove([atIndex = -1]) {
    if (atIndex == -1) atIndex = this._views.length - 1;
    var view = this.detach(atIndex);
    view.dehydrate();
    this.defaultProtoView.returnToPool(view);
  }
  View detach([atIndex = -1]) {
    if (atIndex == -1) atIndex = this._views.length - 1;
    var detachedView = this.get(atIndex);
    ListWrapper.removeAt(this._views, atIndex);
    if (isBlank(this._lightDom)) {
      ViewContainer.removeViewNodesFromParent(
          this.templateElement.parentNode, detachedView);
    } else {
      this._lightDom.redistribute();
    }
    detachedView.changeDetector.remove();
    this._unlinkElementInjectors(detachedView);
    return detachedView;
  }
  contentTagContainers() {
    return this._views;
  }
  List<Node> nodes() {
    var r = [];
    for (var i = 0; i < this._views.length; ++i) {
      r = ListWrapper.concat(r, this._views[i].nodes);
    }
    return r;
  }
  _linkElementInjectors(view) {
    for (var i = 0; i < view.rootElementInjectors.length; ++i) {
      view.rootElementInjectors[i].parent = this.elementInjector;
    }
  }
  _unlinkElementInjectors(view) {
    for (var i = 0; i < view.rootElementInjectors.length; ++i) {
      view.rootElementInjectors[i].parent = null;
    }
  }
  static moveViewNodesAfterSibling(sibling, view) {
    for (var i = view.nodes.length - 1; i >= 0; --i) {
      DOM.insertAfter(sibling, view.nodes[i]);
    }
  }
  static removeViewNodesFromParent(parent, view) {
    for (var i = view.nodes.length - 1; i >= 0; --i) {
      DOM.removeChild(parent, view.nodes[i]);
    }
  }
}
