library angular2.src.core.events.event_manager_dart;

import 'package:angular2/src/facade/lang.dart'
    show isBlank, BaseException, isPresent;
import 'package:angular2/src/facade/dom.dart' show DOM, Element;
import 'package:angular2/src/facade/collection.dart'
    show List, ListWrapper, MapWrapper;
import 'package:angular2/src/core/zone/vm_turn_zone.dart' show VmTurnZone;

class EventManager {
  List<EventManagerPlugin> _plugins;
  VmTurnZone _zone;
  EventManager(List<EventManagerPlugin> plugins, VmTurnZone zone) {
    this._zone = zone;
    this._plugins = plugins;
    for (var i = 0; i < plugins.length; i++) {
      plugins[i].manager = this;
    }
  }
  addEventListener(Element element, String eventName, Function handler) {
    var plugin = this._findPluginFor(eventName);
    if (isPresent(plugin)) {
      plugin.addEventListener(element, eventName, handler);
    } else {
      this._addNativeEventListener(element, eventName, handler);
    }
  }
  VmTurnZone getZone() {
    return this._zone;
  }
  EventManagerPlugin _findPluginFor(String eventName) {
    var plugins = this._plugins;
    for (var i = 0; i < plugins.length; i++) {
      var plugin = plugins[i];
      if (plugin.supports(eventName)) {
        return plugin;
      }
    }
    return null;
  }
  _addNativeEventListener(Element element, String eventName, Function handler) {
    this._zone.runOutsideAngular(() {
      DOM.on(element, eventName, (event) {
        if (identical(event.target, element)) {
          this._zone.run(() {
            handler(event);
          });
        }
      });
    });
  }
}
class EventManagerPlugin {
  EventManager manager;
  bool supports(String eventName) {
    return false;
  }
  addEventListener(Element element, String eventName, Function handler) {
    throw "not implemented";
  }
}
