library angular2.src.di.binding_dart;

import 'package:angular2/src/facade/lang.dart'
    show FIELD, Type, isBlank, isPresent;
import 'package:angular2/src/facade/collection.dart'
    show List, MapWrapper, ListWrapper;
import 'package:angular2/src/reflection/reflection.dart' show reflector;
import './key.dart' show Key;
import './annotations.dart'
    show Inject, InjectLazy, InjectPromise, DependencyAnnotation;
import './exceptions.dart' show NoAnnotationError;

class Dependency {
  Key key;
  bool asPromise;
  bool lazy;
  List properties;
  Dependency(Key key, bool asPromise, bool lazy, List properties) {
    this.key = key;
    this.asPromise = asPromise;
    this.lazy = lazy;
    this.properties = properties;
  }
}
class Binding {
  Key key;
  Function factory;
  List dependencies;
  bool providedAsPromise;
  Binding(
      Key key, Function factory, List dependencies, bool providedAsPromise) {
    this.key = key;
    this.factory = factory;
    this.dependencies = dependencies;
    this.providedAsPromise = providedAsPromise;
  }
}
BindingBuilder bind(token) {
  return new BindingBuilder(token);
}
class BindingBuilder {
  var token;
  BindingBuilder(token) {
    this.token = token;
  }
  Binding toClass(Type type) {
    return new Binding(Key.get(this.token), reflector.factory(type),
        _dependenciesFor(type), false);
  }
  Binding toValue(value) {
    return new Binding(Key.get(this.token), () {
      return value;
    }, [], false);
  }
  Binding toFactory(Function factoryFunction, [List dependencies = null]) {
    return new Binding(Key.get(this.token), factoryFunction,
        this._constructDependencies(factoryFunction, dependencies), false);
  }
  Binding toAsyncFactory(Function factoryFunction, [List dependencies = null]) {
    return new Binding(Key.get(this.token), factoryFunction,
        this._constructDependencies(factoryFunction, dependencies), true);
  }
  _constructDependencies(Function factoryFunction, List dependencies) {
    return isBlank(dependencies)
        ? _dependenciesFor(factoryFunction)
        : ListWrapper.map(dependencies, (t) {
      return new Dependency(Key.get(t), false, false, []);
    });
  }
}
List _dependenciesFor(typeOrFunc) {
  var params = reflector.parameters(typeOrFunc);
  if (isBlank(params)) return [];
  if (ListWrapper.any(params, (p) {
    return isBlank(p);
  })) throw new NoAnnotationError(typeOrFunc);
  return ListWrapper.map(params, (p) {
    return _extractToken(typeOrFunc, p);
  });
}
_extractToken(typeOrFunc, annotations) {
  var type;
  var depProps = [];
  for (var i = 0; i < annotations.length; ++i) {
    var paramAnnotation = annotations[i];
    if (paramAnnotation is Type) {
      type = paramAnnotation;
    } else if (paramAnnotation is Inject) {
      return _createDependency(paramAnnotation.token, false, false, []);
    } else if (paramAnnotation is InjectPromise) {
      return _createDependency(paramAnnotation.token, true, false, []);
    } else if (paramAnnotation is InjectLazy) {
      return _createDependency(paramAnnotation.token, false, true, []);
    } else if (paramAnnotation is DependencyAnnotation) {
      ListWrapper.push(depProps, paramAnnotation);
    }
  }
  if (isPresent(type)) {
    return _createDependency(type, false, false, depProps);
  } else {
    throw new NoAnnotationError(typeOrFunc);
  }
}
Dependency _createDependency(token, asPromise, lazy, depProps) {
  return new Dependency(Key.get(token), asPromise, lazy, depProps);
}
