library angular2.src.directives.foreach_dart;

import 'package:angular2/src/core/annotations/annotations.dart'
    show Viewport, onChange;
import 'package:angular2/src/core/compiler/interfaces.dart' show OnChange;
import 'package:angular2/src/core/compiler/view_container.dart'
    show ViewContainer;
import 'package:angular2/src/core/compiler/view.dart' show View;
import 'package:angular2/src/facade/lang.dart' show isPresent, isBlank;
import 'package:angular2/src/facade/collection.dart' show ListWrapper;

@Viewport(
    selector: '[foreach][in]',
    lifecycle: const [onChange],
    bind: const {'in': 'iterable[]'})
class Foreach extends OnChange {
  ViewContainer viewContainer;
  var iterable;
  Foreach(ViewContainer viewContainer) : super() {
    this.viewContainer = viewContainer;
  }
  onChange(changes) {
    var iteratorChanges = changes['iterable'];
    if (isBlank(iteratorChanges) || isBlank(iteratorChanges.currentValue)) {
      this.viewContainer.clear();
      return;
    }
    var recordViewTuples = [];
    iteratorChanges.currentValue.forEachRemovedItem((removedRecord) {
      return ListWrapper.push(
          recordViewTuples, new RecordViewTuple(removedRecord, null));
    });
    iteratorChanges.currentValue.forEachMovedItem((movedRecord) {
      return ListWrapper.push(
          recordViewTuples, new RecordViewTuple(movedRecord, null));
    });
    var insertTuples = Foreach.bulkRemove(recordViewTuples, this.viewContainer);
    iteratorChanges.currentValue.forEachAddedItem((addedRecord) {
      return ListWrapper.push(
          insertTuples, new RecordViewTuple(addedRecord, null));
    });
    Foreach.bulkInsert(insertTuples, this.viewContainer);
    for (var i = 0; i < insertTuples.length; i++) {
      this.perViewChange(insertTuples[i].view, insertTuples[i].record);
    }
  }
  perViewChange(view, record) {
    view.setLocal('\$implicit', record.item);
    view.setLocal('index', record.currentIndex);
  }
  static bulkRemove(tuples, viewContainer) {
    tuples.sort((a, b) {
      return a.record.previousIndex - b.record.previousIndex;
    });
    var movedTuples = [];
    for (var i = tuples.length - 1; i >= 0; i--) {
      var tuple = tuples[i];
      if (isPresent(tuple.record.currentIndex)) {
        tuple.view = viewContainer.detach(tuple.record.previousIndex);
        ListWrapper.push(movedTuples, tuple);
      } else {
        viewContainer.remove(tuple.record.previousIndex);
      }
    }
    return movedTuples;
  }
  static bulkInsert(tuples, viewContainer) {
    tuples.sort((a, b) {
      return a.record.currentIndex - b.record.currentIndex;
    });
    for (var i = 0; i < tuples.length; i++) {
      var tuple = tuples[i];
      if (isPresent(tuple.view)) {
        viewContainer.insert(tuple.view, tuple.record.currentIndex);
      } else {
        tuple.view = viewContainer.create(tuple.record.currentIndex);
      }
    }
    return tuples;
  }
}
class RecordViewTuple {
  View view;
  dynamic record;
  RecordViewTuple(record, view) {
    this.record = record;
    this.view = view;
  }
}
