library angular2.src.forms.directives_dart;

import 'package:angular2/core.dart'
    show Template, Component, Decorator, NgElement, Ancestor, onChange;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/src/facade/lang.dart' show isBlank, isPresent, CONST;
import 'package:angular2/src/facade/collection.dart'
    show StringMapWrapper, ListWrapper;
import './model.dart' show ControlGroup, Control;

class ControlGroupDirectiveBase {
  addDirective(directive) {}
  Control findControl(String name) {
    return null;
  }
}
@CONST()
class ControlValueAccessor {
  readValue(el) {}
  writeValue(el, value) {}
}
@CONST()
class DefaultControlValueAccessor extends ControlValueAccessor {
  DefaultControlValueAccessor() : super() {}
  readValue(el) {
    return el.value;
  }
  writeValue(el, value) {
    el.value = value;
  }
}
@CONST()
class CheckboxControlValueAccessor extends ControlValueAccessor {
  CheckboxControlValueAccessor() : super() {}
  bool readValue(el) {
    return el.checked;
  }
  writeValue(el, bool value) {
    el.checked = value;
  }
}
var controlValueAccessors = {
  "checkbox": new CheckboxControlValueAccessor(),
  "text": new DefaultControlValueAccessor()
};
ControlValueAccessor controlValueAccessorFor(String controlType) {
  var accessor = StringMapWrapper.get(controlValueAccessors, controlType);
  if (isPresent(accessor)) {
    return accessor;
  } else {
    return StringMapWrapper.get(controlValueAccessors, "text");
  }
}
class ControlDirectiveBase {
  ControlGroupDirectiveBase _groupDecorator;
  NgElement _el;
  String controlName;
  String type;
  ControlValueAccessor valueAccessor;
  ControlDirectiveBase(groupDecorator, NgElement el) {
    this._groupDecorator = groupDecorator;
    this._el = el;
  }
  _initialize() {
    if (isBlank(this.valueAccessor)) {
      this.valueAccessor = controlValueAccessorFor(this.type);
    }
    this._groupDecorator.addDirective(this);
    this._updateDomValue();
    DOM.on(this._el.domElement, "change", (_) {
      return this._updateControlValue();
    });
  }
  _updateDomValue() {
    this.valueAccessor.writeValue(this._el.domElement, this._control().value);
  }
  _updateControlValue() {
    this._control().value = this.valueAccessor.readValue(this._el.domElement);
  }
  _control() {
    return this._groupDecorator.findControl(this.controlName);
  }
}
@Decorator(
    lifecycle: const [onChange],
    selector: '[control-name]',
    bind: const {'control-name': 'controlName', 'type': 'type'})
class ControlNameDirective extends ControlDirectiveBase {
  ControlNameDirective(
      @Ancestor() ControlGroupDirective groupDecorator, NgElement el)
      : super(groupDecorator, el) {}
  onChange(_) {
    this._initialize();
  }
}
@Decorator(
    lifecycle: const [onChange],
    selector: '[control]',
    bind: const {'control': 'controlName', 'type': 'type'})
class ControlDirective extends ControlDirectiveBase {
  ControlDirective(
      @Ancestor() NewControlGroupDirective groupDecorator, NgElement el)
      : super(groupDecorator, el) {}
  onChange(_) {
    this._initialize();
  }
}
@Decorator(
    selector: '[control-group]', bind: const {'control-group': 'controlGroup'})
class ControlGroupDirective extends ControlGroupDirectiveBase {
  ControlGroup _controlGroup;
  List<ControlNameDirective> _directives;
  ControlGroupDirective() : super() {
    this._directives = ListWrapper.create();
  }
  set controlGroup(ControlGroup controlGroup) {
    this._controlGroup = controlGroup;
    ListWrapper.forEach(this._directives, (cd) {
      return cd._updateDomValue();
    });
  }
  addDirective(ControlNameDirective c) {
    ListWrapper.push(this._directives, c);
  }
  Control findControl(String name) {
    return this._controlGroup.controls[name];
  }
}
@Component(
    selector: '[new-control-group]',
    bind: const {'new-control-group': 'initData'})
@Template(inline: '<content>')
class NewControlGroupDirective extends ControlGroupDirectiveBase {
  dynamic _initData;
  ControlGroup _controlGroup;
  List<ControlNameDirective> _directives;
  NewControlGroupDirective() : super() {
    this._directives = ListWrapper.create();
  }
  set initData(value) {
    this._initData = value;
  }
  addDirective(ControlDirective c) {
    ListWrapper.push(this._directives, c);
    this._controlGroup = null;
  }
  Control findControl(String name) {
    if (isBlank(this._controlGroup)) {
      this._controlGroup = this._createControlGroup();
    }
    return this._controlGroup.controls[name];
  }
  ControlGroup _createControlGroup() {
    var controls = ListWrapper.reduce(this._directives, (memo, cd) {
      var initControlValue = this._initData[cd.controlName];
      memo[cd.controlName] = new Control(initControlValue);
      return memo;
    }, {});
    return new ControlGroup(controls);
  }
  get value {
    return this._controlGroup.value;
  }
}
var FormDirectives = [
  ControlGroupDirective,
  ControlNameDirective,
  ControlDirective,
  NewControlGroupDirective
];
