library angular2.test.core.application_spec_dart;

import 'package:angular2/test_lib.dart'
    show describe, ddescribe, it, iit, xit, xdescribe, expect, beforeEach;
import 'package:angular2/src/core/application.dart'
    show bootstrap, appDocumentToken, appElementToken;
import 'package:angular2/src/core/annotations/annotations.dart' show Component;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/src/facade/collection.dart' show ListWrapper;
import 'package:angular2/src/facade/async.dart' show PromiseWrapper;
import 'package:angular2/di.dart' show bind, Inject;
import 'package:angular2/src/core/annotations/template.dart' show Template;
import 'package:angular2/src/core/life_cycle/life_cycle.dart' show LifeCycle;

@Component(selector: 'hello-app')
@Template(inline: '{{greeting}} world!')
class HelloRootCmp {
  String greeting;
  HelloRootCmp() {
    this.greeting = 'hello';
  }
}
@Component(selector: 'hello-app-2')
@Template(inline: '{{greeting}} world, again!')
class HelloRootCmp2 {
  String greeting;
  HelloRootCmp2() {
    this.greeting = 'hello';
  }
}
@Component(selector: 'hello-app')
@Template(inline: '')
class HelloRootCmp3 {
  var appBinding;
  HelloRootCmp3(@Inject("appBinding") appBinding) {
    this.appBinding = appBinding;
  }
}
@Component(selector: 'hello-app')
@Template(inline: '')
class HelloRootCmp4 {
  var lc;
  HelloRootCmp4(@Inject(LifeCycle) lc) {
    this.lc = lc;
  }
}
main() {
  var fakeDoc;
  var el;
  var el2;
  var testBindings;
  beforeEach(() {
    fakeDoc = DOM.createHtmlDocument();
    el = DOM.createElement('hello-app', fakeDoc);
    el2 = DOM.createElement('hello-app-2', fakeDoc);
    DOM.appendChild(fakeDoc.body, el);
    DOM.appendChild(fakeDoc.body, el2);
    testBindings = [bind(appDocumentToken).toValue(fakeDoc)];
  });
  describe('bootstrap factory method', () {
    it('should throw if no element is found', (done) {
      var injectorPromise = bootstrap(HelloRootCmp, [], (e, t) {
        throw e;
      });
      PromiseWrapper.then(injectorPromise, null, (reason) {
        expect(reason.message).toContain(
            'The app selector "hello-app" did not match any elements');
        done();
      });
    });
    it('should create an injector promise', () {
      var injectorPromise = bootstrap(HelloRootCmp, testBindings);
      expect(injectorPromise).not.toBe(null);
    });
    it('should resolve an injector promise and contain bindings', (done) {
      var injectorPromise = bootstrap(HelloRootCmp, testBindings);
      injectorPromise.then((injector) {
        expect(injector.get(appElementToken)).toBe(el);
        done();
      });
    });
    it('should provide the application component in the injector', (done) {
      var injectorPromise = bootstrap(HelloRootCmp, testBindings);
      injectorPromise.then((injector) {
        expect(injector.get(HelloRootCmp)).toBeAnInstanceOf(HelloRootCmp);
        done();
      });
    });
    it('should display hello world', (done) {
      var injectorPromise = bootstrap(HelloRootCmp, testBindings);
      injectorPromise.then((injector) {
        expect(injector.get(appElementToken).shadowRoot.childNodes[0].nodeValue)
            .toEqual('hello world!');
        done();
      });
    });
    it('should support multiple calls to bootstrap', (done) {
      var injectorPromise1 = bootstrap(HelloRootCmp, testBindings);
      var injectorPromise2 = bootstrap(HelloRootCmp2, testBindings);
      PromiseWrapper.all([injectorPromise1, injectorPromise2]).then(
          (injectors) {
        expect(injectors[0].get(appElementToken).shadowRoot.childNodes[
            0].nodeValue).toEqual('hello world!');
        expect(injectors[1].get(appElementToken).shadowRoot.childNodes[
            0].nodeValue).toEqual('hello world, again!');
        done();
      });
    });
    it("should make the provided bindings available to the application component",
        (done) {
      var injectorPromise = bootstrap(HelloRootCmp3, [
        testBindings,
        bind("appBinding").toValue("BoundValue")
      ]);
      injectorPromise.then((injector) {
        expect(injector.get(HelloRootCmp3).appBinding).toEqual("BoundValue");
        done();
      });
    });
    it("should avoid cyclic dependencies when root component requires Lifecycle through DI",
        (done) {
      var injectorPromise = bootstrap(HelloRootCmp4, testBindings);
      injectorPromise.then((injector) {
        expect(injector.get(HelloRootCmp4).lc).toBe(injector.get(LifeCycle));
        done();
      });
    });
  });
}
