library angular2.test.core.compiler.pipeline.element_binder_builder_spec_dart;

import 'package:angular2/test_lib.dart'
    show describe, beforeEach, it, expect, iit, ddescribe, el;
import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/src/facade/collection.dart'
    show ListWrapper, MapWrapper;
import 'package:angular2/src/core/compiler/pipeline/element_binder_builder.dart'
    show ElementBinderBuilder;
import 'package:angular2/src/core/compiler/pipeline/compile_pipeline.dart'
    show CompilePipeline;
import 'package:angular2/src/core/compiler/pipeline/compile_element.dart'
    show CompileElement;
import 'package:angular2/src/core/compiler/pipeline/compile_step.dart'
    show CompileStep;
import 'package:angular2/src/core/compiler/pipeline/compile_control.dart'
    show CompileControl;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show NativeShadowDomStrategy;
import 'package:angular2/src/core/annotations/annotations.dart'
    show Decorator, Component, Viewport;
import 'package:angular2/src/core/compiler/view.dart'
    show ProtoView, ElementPropertyMemento, DirectivePropertyMemento;
import 'package:angular2/src/core/compiler/element_injector.dart'
    show ProtoElementInjector;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart'
    show DirectiveMetadataReader;
import 'package:angular2/change_detection.dart'
    show ChangeDetector, Lexer, Parser, DynamicProtoChangeDetector;
import 'package:angular2/di.dart' show Injector;

main() {
  describe('ElementBinderBuilder', () {
    var evalContext;
    var view;
    var changeDetector;
    createPipeline({textNodeBindings, propertyBindings, eventBindings,
        directives, protoElementInjector}) {
      var reflector = new DirectiveMetadataReader();
      var parser = new Parser(new Lexer());
      return new CompilePipeline([
        new MockStep((parent, current, control) {
          var hasBinding = false;
          if (isPresent(current.element.getAttribute('text-binding'))) {
            MapWrapper.forEach(textNodeBindings, (v, k) {
              current.addTextNodeBinding(k, parser.parseBinding(v, null));
            });
            hasBinding = true;
          }
          if (isPresent(current.element.getAttribute('prop-binding'))) {
            if (isPresent(propertyBindings)) {
              MapWrapper.forEach(propertyBindings, (v, k) {
                current.addPropertyBinding(k, parser.parseBinding(v, null));
              });
            }
            hasBinding = true;
          }
          if (isPresent(current.element.getAttribute('event-binding'))) {
            MapWrapper.forEach(eventBindings, (v, k) {
              current.addEventBinding(k, parser.parseAction(v, null));
            });
            hasBinding = true;
          }
          if (isPresent(current.element.getAttribute('directives'))) {
            hasBinding = true;
            for (var i = 0; i < directives.length; i++) {
              var dirMetadata = reflector.read(directives[i]);
              current.addDirective(dirMetadata);
            }
          }
          if (hasBinding) {
            current.hasBindings = true;
            DOM.addClass(current.element, 'ng-binding');
          }
          if (isPresent(protoElementInjector) &&
              (isPresent(current.element.getAttribute('text-binding')) ||
                  isPresent(current.element.getAttribute('prop-binding')) ||
                  isPresent(current.element.getAttribute('directives')) ||
                  isPresent(current.element.getAttribute('event-binding')))) {
            current.inheritedProtoElementInjector = protoElementInjector;
          }
          if (isPresent(current.element.getAttribute('viewroot'))) {
            current.isViewRoot = true;
            current.inheritedProtoView = new ProtoView(current.element,
                new DynamicProtoChangeDetector(),
                new NativeShadowDomStrategy());
          } else if (isPresent(parent)) {
            current.inheritedProtoView = parent.inheritedProtoView;
          }
        }),
        new ElementBinderBuilder(parser, null)
      ]);
    }
    instantiateView(protoView) {
      evalContext = new Context();
      view = protoView.instantiate(null, null);
      view.hydrate(new Injector([]), null, evalContext);
      changeDetector = view.changeDetector;
    }
    it('should not create an ElementBinder for elements that have no bindings',
        () {
      var pipeline = createPipeline();
      var results = pipeline.process(el('<div viewroot><span></span></div>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders.length).toBe(0);
    });
    it('should create an ElementBinder for elements that have bindings', () {
      var pipeline = createPipeline();
      var results = pipeline.process(
          el('<div viewroot prop-binding><span prop-binding></span></div>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders.length).toBe(2);
      expect(pv.elementBinders[1]).not.toBe(pv.elementBinders[0]);
    });
    it('should inherit ElementBinders to children that have no bindings', () {
      var pipeline = createPipeline();
      var results = pipeline
          .process(el('<div viewroot prop-binding><span></span></div>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders.length).toBe(1);
      expect(results[0].inheritedElementBinder)
          .toBe(results[1].inheritedElementBinder);
    });
    it('should store the current protoElementInjector', () {
      var directives = [SomeDecoratorDirective];
      var protoElementInjector = new ProtoElementInjector(null, 0, directives);
      var pipeline = createPipeline(
          protoElementInjector: protoElementInjector, directives: directives);
      var results = pipeline.process(el('<div viewroot directives></div>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders[0].protoElementInjector)
          .toBe(protoElementInjector);
    });
    it('should not store the parent protoElementInjector', () {
      var directives = [SomeDecoratorDirective];
      var eventBindings = MapWrapper.createFromStringMap({'event1': '1+1'});
      var pipeline =
          createPipeline(directives: directives, eventBindings: eventBindings);
      var results = pipeline.process(
          el('<div viewroot directives><div event-binding></div></div>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders[1].protoElementInjector).toBeNull();
    });
    it('should store the component directive', () {
      var directives = [SomeComponentDirective];
      var pipeline =
          createPipeline(protoElementInjector: null, directives: directives);
      var results = pipeline.process(el('<div viewroot directives></div>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders[0].componentDirective.type)
          .toBe(SomeComponentDirective);
    });
    it('should store the template directive', () {
      var directives = [SomeViewportDirective];
      var pipeline =
          createPipeline(protoElementInjector: null, directives: directives);
      var results = pipeline.process(el('<div viewroot directives></div>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders[0].viewportDirective.type)
          .toBe(SomeViewportDirective);
    });
    it('should bind text nodes', () {
      var textNodeBindings = MapWrapper.create();
      MapWrapper.set(textNodeBindings, 0, 'prop1');
      MapWrapper.set(textNodeBindings, 2, 'prop2');
      var pipeline = createPipeline(textNodeBindings: textNodeBindings);
      var results = pipeline.process(
          el('<div viewroot text-binding>{{}}<span></span>{{}}</div>'));
      var pv = results[0].inheritedProtoView;
      expect(sortArr(pv.elementBinders[0].textNodeIndices)).toEqual([0, 2]);
      instantiateView(pv);
      evalContext.prop1 = 'a';
      evalContext.prop2 = 'b';
      changeDetector.detectChanges();
      expect(view.nodes[0].childNodes[0].nodeValue).toEqual('a');
      expect(view.nodes[0].childNodes[2].nodeValue).toEqual('b');
    });
    it('should bind element properties', () {
      var propertyBindings =
          MapWrapper.createFromStringMap({'value': 'prop1', 'hidden': 'prop2'});
      var pipeline = createPipeline(propertyBindings: propertyBindings);
      var results = pipeline.process(el('<input viewroot prop-binding>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders[0].hasElementPropertyBindings).toBe(true);
      instantiateView(pv);
      evalContext.prop1 = 'a';
      evalContext.prop2 = false;
      changeDetector.detectChanges();
      expect(view.nodes[0].value).toEqual('a');
      expect(view.nodes[0].hidden).toEqual(false);
    });
    it('should bind to aria-* attributes when exp evaluates to strings', () {
      var propertyBindings =
          MapWrapper.createFromStringMap({'aria-label': 'prop1'});
      var pipeline = createPipeline(propertyBindings: propertyBindings);
      var results = pipeline.process(el('<div viewroot prop-binding></div>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders[0].hasElementPropertyBindings).toBe(true);
      instantiateView(pv);
      evalContext.prop1 = 'some label';
      changeDetector.detectChanges();
      expect(DOM.getAttribute(view.nodes[0], 'aria-label'))
          .toEqual('some label');
      evalContext.prop1 = 'some other label';
      changeDetector.detectChanges();
      expect(DOM.getAttribute(view.nodes[0], 'aria-label'))
          .toEqual('some other label');
      evalContext.prop1 = null;
      changeDetector.detectChanges();
      expect(DOM.getAttribute(view.nodes[0], 'aria-label')).toBeNull();
    });
    it('should bind to aria-* attributes when exp evaluates to booleans', () {
      var propertyBindings =
          MapWrapper.createFromStringMap({'aria-busy': 'prop1'});
      var pipeline = createPipeline(propertyBindings: propertyBindings);
      var results = pipeline.process(el('<div viewroot prop-binding></div>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders[0].hasElementPropertyBindings).toBe(true);
      instantiateView(pv);
      evalContext.prop1 = true;
      changeDetector.detectChanges();
      expect(DOM.getAttribute(view.nodes[0], 'aria-busy')).toEqual('true');
      evalContext.prop1 = false;
      changeDetector.detectChanges();
      expect(DOM.getAttribute(view.nodes[0], 'aria-busy')).toEqual('false');
    });
    it('should bind to ARIA role attribute', () {
      var propertyBindings = MapWrapper.createFromStringMap({'role': 'prop1'});
      var pipeline = createPipeline(propertyBindings: propertyBindings);
      var results = pipeline.process(el('<div viewroot prop-binding></div>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders[0].hasElementPropertyBindings).toBe(true);
      instantiateView(pv);
      evalContext.prop1 = 'alert';
      changeDetector.detectChanges();
      expect(DOM.getAttribute(view.nodes[0], 'role')).toEqual('alert');
      evalContext.prop1 = 'alertdialog';
      changeDetector.detectChanges();
      expect(DOM.getAttribute(view.nodes[0], 'role')).toEqual('alertdialog');
      evalContext.prop1 = null;
      changeDetector.detectChanges();
      expect(DOM.getAttribute(view.nodes[0], 'role')).toBeNull();
    });
    it('should throw for a non-string ARIA role', () {
      var propertyBindings = MapWrapper.createFromStringMap({'role': 'prop1'});
      var pipeline = createPipeline(propertyBindings: propertyBindings);
      var results = pipeline.process(el('<div viewroot prop-binding></div>'));
      var pv = results[0].inheritedProtoView;
      instantiateView(pv);
      expect(() {
        evalContext.prop1 = 1;
        changeDetector.detectChanges();
      }).toThrowError(
          "Invalid role attribute, only string values are allowed, got '1'");
    });
    it('should bind class with a dot', () {
      var propertyBindings =
          MapWrapper.createFromStringMap({'class.bar': 'prop1'});
      var pipeline = createPipeline(propertyBindings: propertyBindings);
      var results =
          pipeline.process(el('<input class="foo" viewroot prop-binding>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders[0].hasElementPropertyBindings).toBe(true);
      instantiateView(pv);
      evalContext.prop1 = true;
      changeDetector.detectChanges();
      expect(view.nodes[0].className).toEqual('foo ng-binding bar');
      evalContext.prop1 = false;
      changeDetector.detectChanges();
      expect(view.nodes[0].className).toEqual('foo ng-binding');
    });
    it('should bind style with a dot', () {
      var propertyBindings =
          MapWrapper.createFromStringMap({'style.color': 'prop1'});
      var pipeline = createPipeline(propertyBindings: propertyBindings);
      var results = pipeline.process(el('<div viewroot prop-binding>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders[0].hasElementPropertyBindings).toBe(true);
      instantiateView(pv);
      evalContext.prop1 = 'red';
      changeDetector.detectChanges();
      expect(view.nodes[0].style.color).toEqual('red');
      evalContext.prop1 = 'blue';
      changeDetector.detectChanges();
      expect(view.nodes[0].style.color).toEqual('blue');
    });
    it('should bind style with a dot and suffix', () {
      var propertyBindings =
          MapWrapper.createFromStringMap({'style.font-size.px': 'prop1'});
      var pipeline = createPipeline(propertyBindings: propertyBindings);
      var results = pipeline.process(el('<div viewroot prop-binding>'));
      var pv = results[0].inheritedProtoView;
      expect(pv.elementBinders[0].hasElementPropertyBindings).toBe(true);
      instantiateView(pv);
      evalContext.prop1 = 10;
      changeDetector.detectChanges();
      expect(DOM.getStyle(view.nodes[0], 'font-size')).toEqual('10px');
      evalContext.prop1 = 20;
      changeDetector.detectChanges();
      expect(DOM.getStyle(view.nodes[0], 'font-size')).toEqual('20px');
      evalContext.prop1 = null;
      changeDetector.detectChanges();
      expect(DOM.getStyle(view.nodes[0], 'font-size')).toEqual('');
    });
    it('should bind events', () {
      var eventBindings = MapWrapper.createFromStringMap({'event1': '1+1'});
      var pipeline = createPipeline(eventBindings: eventBindings);
      var results = pipeline.process(el('<div viewroot event-binding></div>'));
      var pv = results[0].inheritedProtoView;
      var ast = MapWrapper.get(pv.elementBinders[0].events, 'event1');
      expect(ast.eval(null)).toBe(2);
    });
    it('should bind directive properties', () {
      var propertyBindings = MapWrapper.createFromStringMap({
        'boundprop1': 'prop1',
        'boundprop2': 'prop2',
        'boundprop3': 'prop3'
      });
      var directives = [
        SomeComponentDirectiveWithBinding,
        SomeViewportDirectiveWithBinding,
        SomeDecoratorDirectiveWith2Bindings
      ];
      var protoElementInjector =
          new ProtoElementInjector(null, 0, directives, true);
      var pipeline = createPipeline(
          propertyBindings: propertyBindings,
          directives: directives,
          protoElementInjector: protoElementInjector);
      var results =
          pipeline.process(el('<div viewroot prop-binding directives></div>'));
      var pv = results[0].inheritedProtoView;
      results[0].inheritedElementBinder.nestedProtoView = new ProtoView(
          el('<div></div>'), new DynamicProtoChangeDetector(),
          new NativeShadowDomStrategy());
      instantiateView(pv);
      evalContext.prop1 = 'a';
      evalContext.prop2 = 'b';
      evalContext.prop3 = 'c';
      changeDetector.detectChanges();
      expect(view.elementInjectors[0]
          .get(SomeDecoratorDirectiveWith2Bindings).decorProp).toBe('a');
      expect(view.elementInjectors[0]
          .get(SomeDecoratorDirectiveWith2Bindings).decorProp2).toBe('b');
      expect(view.elementInjectors[0]
          .get(SomeViewportDirectiveWithBinding).templProp).toBe('b');
      expect(view.elementInjectors[0]
          .get(SomeComponentDirectiveWithBinding).compProp).toBe('c');
    });
    it('should bind directive properties for sibling elements', () {
      var propertyBindings =
          MapWrapper.createFromStringMap({'boundprop1': 'prop1'});
      var directives = [SomeDecoratorDirectiveWithBinding];
      var protoElementInjector = new ProtoElementInjector(null, 0, directives);
      var pipeline = createPipeline(
          propertyBindings: propertyBindings,
          directives: directives,
          protoElementInjector: protoElementInjector);
      var results = pipeline.process(el(
          '<div viewroot><div prop-binding directives>' +
              '</div><div prop-binding directives></div></div>'));
      var pv = results[0].inheritedProtoView;
      instantiateView(pv);
      evalContext.prop1 = 'a';
      changeDetector.detectChanges();
      expect(view.elementInjectors[1]
          .get(SomeDecoratorDirectiveWithBinding).decorProp).toBe('a');
    });
    it('should bind to string literals', () {
      var directives = [SomeDecoratorDirectiveWithBinding];
      var protoElementInjector = new ProtoElementInjector(null, 0, directives);
      var pipeline = createPipeline(
          directives: directives, protoElementInjector: protoElementInjector);
      var results = pipeline
          .process(el('<div viewroot directives boundprop1="foo"></div>'));
      var pv = results[0].inheritedProtoView;
      instantiateView(pv);
      changeDetector.detectChanges();
      expect(view.elementInjectors[0]
          .get(SomeDecoratorDirectiveWithBinding).decorProp).toEqual('foo');
    });
    describe('errors', () {
      it('should throw if there is no element property bindings for a directive property binding',
          () {
        var pipeline = createPipeline(
            propertyBindings: MapWrapper.create(),
            directives: [SomeDecoratorDirectiveWithBinding]);
        expect(() {
          pipeline.process(el('<div viewroot prop-binding directives>'));
        }).toThrowError(
            "No element binding found for property 'boundprop1' which is required by directive 'SomeDecoratorDirectiveWithBinding'");
      });
    });
  });
}
@Decorator()
class SomeDecoratorDirective {}
@Decorator(bind: const {'boundprop1': 'decorProp'})
class SomeDecoratorDirectiveWithBinding {
  var decorProp;
  var decorProp2;
  SomeDecoratorDirectiveWithBinding() {
    this.decorProp = null;
    this.decorProp2 = null;
  }
}
@Decorator(bind: const {'boundprop1': 'decorProp', 'boundprop2': 'decorProp2'})
class SomeDecoratorDirectiveWith2Bindings {
  var decorProp;
  var decorProp2;
  SomeDecoratorDirectiveWith2Bindings() {
    this.decorProp = null;
    this.decorProp2 = null;
  }
}
@Viewport()
class SomeViewportDirective {}
@Viewport(bind: const {'boundprop2': 'templProp'})
class SomeViewportDirectiveWithBinding {
  var templProp;
  SomeViewportDirectiveWithBinding() {
    this.templProp = null;
  }
}
@Component()
class SomeComponentDirective {}
@Component(bind: const {'boundprop3': 'compProp'})
class SomeComponentDirectiveWithBinding {
  var compProp;
  SomeComponentDirectiveWithBinding() {
    this.compProp = null;
  }
}
class Context {
  var prop1;
  var prop2;
  var prop3;
  Context() {
    this.prop1 = null;
    this.prop2 = null;
    this.prop3 = null;
  }
}
class MockStep extends CompileStep {
  Function processClosure;
  MockStep(process) : super() {
    this.processClosure = process;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    this.processClosure(parent, current, control);
  }
}
sortArr(arr) {
  var arr2 = ListWrapper.clone(arr);
  arr2.sort();
  return arr2;
}
