library angular2.test.core.compiler.shadow_dom.light_dom_spec_dart;

import 'package:angular2/test_lib.dart'
    show describe, beforeEach, it, expect, ddescribe, iit, SpyObject, el, proxy;
import 'package:angular2/src/facade/lang.dart' show IMPLEMENTS, isBlank;
import 'package:angular2/src/facade/collection.dart'
    show ListWrapper, MapWrapper;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/src/core/compiler/shadow_dom_emulation/content_tag.dart'
    show Content;
import 'package:angular2/src/core/dom/element.dart' show NgElement;
import 'package:angular2/src/core/compiler/shadow_dom_emulation/light_dom.dart'
    show LightDom;
import 'package:angular2/src/core/compiler/view.dart' show View;
import 'package:angular2/src/core/compiler/view_container.dart'
    show ViewContainer;
import 'package:angular2/src/core/compiler/element_injector.dart'
    show ElementInjector;
import 'package:angular2/change_detection.dart' show ProtoRecordRange;

@proxy
class FakeElementInjector implements ElementInjector {
  var content;
  var viewContainer;
  var element;
  FakeElementInjector([content = null, viewContainer = null, element = null]) {
    this.content = content;
    this.viewContainer = viewContainer;
    this.element = element;
  }
  hasDirective(type) {
    return this.content != null;
  }
  hasPreBuiltObject(type) {
    return this.viewContainer != null;
  }
  forElement(n) {
    return this.element == n;
  }
  get(t) {
    if (identical(t, Content)) return this.content;
    if (identical(t, ViewContainer)) return this.viewContainer;
    return null;
  }
  noSuchMethod(i) {
    super.noSuchMethod(i);
  }
}
@proxy
class FakeView implements View {
  var elementInjectors;
  FakeView([elementInjectors = null]) {
    this.elementInjectors = elementInjectors;
  }
  noSuchMethod(i) {
    super.noSuchMethod(i);
  }
}
@proxy
class FakeViewContainer implements ViewContainer {
  var _nodes;
  var _contentTagContainers;
  FakeViewContainer([nodes = null, views = null]) {
    this._nodes = nodes;
    this._contentTagContainers = views;
  }
  nodes() {
    return this._nodes;
  }
  contentTagContainers() {
    return this._contentTagContainers;
  }
  noSuchMethod(i) {
    super.noSuchMethod(i);
  }
}
@proxy
class FakeContentTag implements Content {
  var select;
  var _nodes;
  FakeContentTag([select = null, nodes = null]) {
    this.select = select;
    this._nodes = nodes;
  }
  insert(nodes) {
    this._nodes = ListWrapper.clone(nodes);
  }
  nodes() {
    return this._nodes;
  }
  noSuchMethod(i) {
    super.noSuchMethod(i);
  }
}
main() {
  describe('LightDom', () {
    var lightDomView;
    beforeEach(() {
      lightDomView = new FakeView([]);
    });
    describe("contentTags", () {
      it("should collect content tags from element injectors", () {
        var tag = new FakeContentTag();
        var shadowDomView = new FakeView([new FakeElementInjector(tag)]);
        var lightDom =
            new LightDom(lightDomView, shadowDomView, el("<div></div>"));
        expect(lightDom.contentTags()).toEqual([tag]);
      });
      it("should collect content tags from ViewContainers", () {
        var tag = new FakeContentTag();
        var vp = new FakeViewContainer(
            null, [new FakeView([new FakeElementInjector(tag, null)])]);
        var shadowDomView = new FakeView([new FakeElementInjector(null, vp)]);
        var lightDom =
            new LightDom(lightDomView, shadowDomView, el("<div></div>"));
        expect(lightDom.contentTags()).toEqual([tag]);
      });
    });
    describe("expanded roots", () {
      it("should contain root nodes", () {
        var lightDomEl = el("<div><a></a></div>");
        var lightDom = new LightDom(lightDomView, new FakeView(), lightDomEl);
        expect(toHtml(lightDom.expandedDomNodes())).toEqual(["<a></a>"]);
      });
      it("should include ViewContainer nodes", () {
        var lightDomEl = el("<div><template></template></div>");
        var lightDomView = new FakeView([
          new FakeElementInjector(null, new FakeViewContainer([el("<a></a>")]),
              DOM.firstChild(lightDomEl))
        ]);
        var lightDom = new LightDom(lightDomView, new FakeView(), lightDomEl);
        expect(toHtml(lightDom.expandedDomNodes())).toEqual(["<a></a>"]);
      });
      it("should include content nodes", () {
        var lightDomEl = el("<div><content></content></div>");
        var lightDomView = new FakeView([
          new FakeElementInjector(new FakeContentTag(null, [el("<a></a>")]),
              null, DOM.firstChild(lightDomEl))
        ]);
        var lightDom = new LightDom(lightDomView, new FakeView(), lightDomEl);
        expect(toHtml(lightDom.expandedDomNodes())).toEqual(["<a></a>"]);
      });
    });
    describe("redistribute", () {
      it("should redistribute nodes between content tags with select property set",
          () {
        var contentA = new FakeContentTag("a");
        var contentB = new FakeContentTag("b");
        var lightDomEl = el("<div><a>1</a><b>2</b><a>3</a></div>");
        var lightDom = new LightDom(lightDomView, new FakeView([
          new FakeElementInjector(contentA, null),
          new FakeElementInjector(contentB, null)
        ]), lightDomEl);
        lightDom.redistribute();
        expect(toHtml(contentA.nodes())).toEqual(["<a>1</a>", "<a>3</a>"]);
        expect(toHtml(contentB.nodes())).toEqual(["<b>2</b>"]);
      });
      it("should support wildcard content tags", () {
        var wildcard = new FakeContentTag(null);
        var contentB = new FakeContentTag("b");
        var lightDomEl = el("<div><a>1</a><b>2</b><a>3</a></div>");
        var lightDom = new LightDom(lightDomView, new FakeView([
          new FakeElementInjector(wildcard, null),
          new FakeElementInjector(contentB, null)
        ]), lightDomEl);
        lightDom.redistribute();
        expect(toHtml(wildcard.nodes()))
            .toEqual(["<a>1</a>", "<b>2</b>", "<a>3</a>"]);
        expect(toHtml(contentB.nodes())).toEqual([]);
      });
    });
  });
}
toHtml(nodes) {
  if (isBlank(nodes)) return [];
  return ListWrapper.map(nodes, DOM.getOuterHTML);
}
