library angular2.test.core.compiler.shadow_dom.shadow_dom_emulation_integration_spec_dart;

import 'package:angular2/test_lib.dart'
    show describe, xit, it, expect, beforeEach, ddescribe, iit, el;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/src/facade/collection.dart'
    show StringMapWrapper, MapWrapper, List;
import 'package:angular2/src/facade/lang.dart' show isPresent, Type;
import 'package:angular2/di.dart' show Injector;
import 'package:angular2/change_detection.dart'
    show Lexer, Parser, ChangeDetector, dynamicChangeDetection;
import 'package:angular2/src/core/compiler/compiler.dart'
    show Compiler, CompilerCache;
import 'package:angular2/src/core/life_cycle/life_cycle.dart' show LifeCycle;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart'
    show DirectiveMetadataReader;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show ShadowDomStrategy, NativeShadowDomStrategy, EmulatedShadowDomStrategy;
import 'package:angular2/src/core/compiler/template_loader.dart'
    show TemplateLoader;
import 'package:angular2/src/core/compiler/template_resolver.dart'
    show TemplateResolver;
import 'package:angular2/src/core/annotations/annotations.dart'
    show Decorator, Component, Viewport;
import 'package:angular2/src/core/annotations/template.dart' show Template;
import 'package:angular2/src/core/compiler/view_container.dart'
    show ViewContainer;

main() {
  describe('integration tests', () {
    StringMapWrapper.forEach({
      "native": new NativeShadowDomStrategy(),
      "emulated": new EmulatedShadowDomStrategy()
    }, (strategy, name) {
      describe('''${name} shadow dom strategy''', () {
        var compiler;
        var tplResolver;
        beforeEach(() {
          tplResolver = new FakeTemplateResolver();
          compiler = new Compiler(dynamicChangeDetection,
              new TemplateLoader(null), new DirectiveMetadataReader(),
              new Parser(new Lexer()), new CompilerCache(), strategy,
              tplResolver);
        });
        compile(template, List<Type> directives, assertions) {
          tplResolver.setTemplate(
              MyComp, new Template(inline: template, directives: directives));
          compiler.compile(MyComp).then(createView).then((view) {
            var lc = new LifeCycle(view.changeDetector, false);
            assertions(view, lc);
          });
        }
        it('should support multiple content tags', (done) {
          var temp = '<multiple-content-tags>' +
              '<div>B</div>' +
              '<div>C</div>' +
              '<div class="left">A</div>' +
              '</multiple-content-tags>';
          compile(temp, [MultipleContentTagsComponent], (view, lc) {
            expect(view.nodes).toHaveText('(A, BC)');
            done();
          });
        });
        it('should redistribute only direct children', (done) {
          var temp = '<multiple-content-tags>' +
              '<div>B<div class="left">A</div></div>' +
              '<div>C</div>' +
              '</multiple-content-tags>';
          compile(temp, [MultipleContentTagsComponent], (view, lc) {
            expect(view.nodes).toHaveText('(, BAC)');
            done();
          });
        });
        it("should redistribute direct child viewcontainers when the light dom changes",
            (done) {
          var temp = '<multiple-content-tags>' +
              '<div><div template="manual" class="left">A</div></div>' +
              '<div>B</div>' +
              '</multiple-content-tags>';
          compile(temp, [
            MultipleContentTagsComponent,
            ManualViewportDirective
          ], (view, lc) {
            var dir = view.elementInjectors[1].get(ManualViewportDirective);
            expect(view.nodes).toHaveText('(, B)');
            dir.show();
            lc.tick();
            expect(view.nodes).toHaveText('(, AB)');
            dir.hide();
            lc.tick();
            expect(view.nodes).toHaveText('(, B)');
            done();
          });
        });
        it("should redistribute when the light dom changes", (done) {
          var temp = '<multiple-content-tags>' +
              '<div template="manual" class="left">A</div>' +
              '<div>B</div>' +
              '</multiple-content-tags>';
          compile(temp, [
            MultipleContentTagsComponent,
            ManualViewportDirective
          ], (view, lc) {
            var dir = view.elementInjectors[1].get(ManualViewportDirective);
            expect(view.nodes).toHaveText('(, B)');
            dir.show();
            lc.tick();
            expect(view.nodes).toHaveText('(A, B)');
            dir.hide();
            lc.tick();
            expect(view.nodes).toHaveText('(, B)');
            done();
          });
        });
        it("should support nested components", (done) {
          var temp = '<outer-with-indirect-nested>' +
              '<div>A</div>' +
              '<div>B</div>' +
              '</outer-with-indirect-nested>';
          compile(temp, [OuterWithIndirectNestedComponent], (view, lc) {
            expect(view.nodes).toHaveText('OUTER(SIMPLE(AB))');
            done();
          });
        });
        it("should support nesting with content being direct child of a nested component",
            (done) {
          var temp = '<outer>' +
              '<div template="manual" class="left">A</div>' +
              '<div>B</div>' +
              '<div>C</div>' +
              '</outer>';
          compile(temp, [OuterComponent, ManualViewportDirective], (view, lc) {
            var dir = view.elementInjectors[1].get(ManualViewportDirective);
            expect(view.nodes).toHaveText('OUTER(INNER(INNERINNER(,BC)))');
            dir.show();
            lc.tick();
            expect(view.nodes).toHaveText('OUTER(INNER(INNERINNER(A,BC)))');
            done();
          });
        });
      });
    });
  });
}
class TestDirectiveMetadataReader extends DirectiveMetadataReader {
  var shadowDomStrategy;
  TestDirectiveMetadataReader(shadowDomStrategy) : super() {
    this.shadowDomStrategy = shadowDomStrategy;
  }
  ShadowDomStrategy parseShadowDomStrategy(Component annotation) {
    return this.shadowDomStrategy;
  }
}
@Viewport(selector: '[manual]')
class ManualViewportDirective {
  var viewContainer;
  ManualViewportDirective(ViewContainer viewContainer) {
    this.viewContainer = viewContainer;
  }
  show() {
    this.viewContainer.create();
  }
  hide() {
    this.viewContainer.remove(0);
  }
}
@Viewport(selector: '[auto]', bind: const {'auto': 'auto'})
class AutoViewportDirective {
  var viewContainer;
  AutoViewportDirective(ViewContainer viewContainer) {
    this.viewContainer = viewContainer;
  }
  set auto(bool newValue) {
    if (newValue) {
      this.viewContainer.create();
    } else {
      this.viewContainer.remove(0);
    }
  }
}
@Component(selector: 'simple')
@Template(inline: 'SIMPLE(<content></content>)')
class Simple {}
@Component(selector: 'multiple-content-tags')
@Template(inline: '(<content select=".left"></content>, <content></content>)')
class MultipleContentTagsComponent {}
@Component(selector: 'conditional-content')
@Template(
    inline: '<div>(<div template="auto: cond"><content select=".left"></content></div>, <content></content>)</div>',
    directives: const [AutoViewportDirective])
class ConditionalContentComponent {
  bool cond;
  ConditionalContentComponent() {
    this.cond = false;
  }
  showLeft() {
    this.cond = true;
  }
  hideLeft() {
    this.cond = false;
  }
}
@Component(selector: 'outer-with-indirect-nested')
@Template(
    inline: 'OUTER(<simple><div><content></content></div></simple>)',
    directives: const [Simple])
class OuterWithIndirectNestedComponent {}
@Component(selector: 'outer')
@Template(
    inline: 'OUTER(<inner><content></content></inner>)',
    directives: const [InnerComponent])
class OuterComponent {}
@Component(selector: 'inner')
@Template(
    inline: 'INNER(<innerinner><content></content></innerinner>)',
    directives: const [InnerInnerComponent])
class InnerComponent {}
@Component(selector: 'innerinner')
@Template(
    inline: 'INNERINNER(<content select=".left"></content>,<content></content>)')
class InnerInnerComponent {}
@Component(selector: 'my-comp')
@Template(
    directives: const [
  MultipleContentTagsComponent,
  ManualViewportDirective,
  ConditionalContentComponent,
  OuterWithIndirectNestedComponent,
  OuterComponent
])
class MyComp {}
createView(pv) {
  var view = pv.instantiate(null, null);
  view.hydrate(new Injector([]), null, {});
  return view;
}
class FakeTemplateResolver extends TemplateResolver {
  Map _cmpTemplates;
  FakeTemplateResolver() : super() {
    this._cmpTemplates = MapWrapper.create();
  }
  setTemplate(Type component, Template template) {
    MapWrapper.set(this._cmpTemplates, component, template);
  }
  Template resolve(Type component) {
    var override = MapWrapper.get(this._cmpTemplates, component);
    if (isPresent(override)) {
      return override;
    }
    return super.resolve(component);
  }
}
