library angular2.test.core.compiler.template_loader_spec_dart;

import 'package:angular2/test_lib.dart'
    show describe, it, expect, beforeEach, ddescribe, iit, xit, el;
import 'package:angular2/src/core/compiler/template_loader.dart'
    show TemplateLoader;
import 'package:angular2/src/core/compiler/template_resolver.dart'
    show TemplateResolver;
import 'package:angular2/src/core/compiler/directive_metadata.dart'
    show DirectiveMetadata;
import 'package:angular2/src/core/annotations/annotations.dart' show Component;
import 'package:angular2/src/core/annotations/template.dart' show Template;
import 'package:angular2/src/facade/async.dart' show PromiseWrapper;
import 'package:angular2/src/facade/lang.dart' show Type, stringify, isPresent;
import 'package:angular2/src/facade/collection.dart' show Map, MapWrapper;
import 'package:angular2/src/mock/xhr_mock.dart' show XHRMock;

main() {
  describe('TemplateLoader', () {
    var loader;
    var xhr;
    beforeEach(() {
      xhr = new XHRMock();
      loader = new TemplateLoader(xhr);
    });
    it('should load inline templates synchronously', () {
      var template = new Template(inline: 'inline template');
      expect(loader.load(template).content).toHaveText('inline template');
    });
    it('should load templates through XHR', (done) {
      xhr.expect('/foo', 'xhr template');
      var template = new Template(url: '/foo');
      loader.load(template).then((el) {
        expect(el.content).toHaveText('xhr template');
        done();
      });
      xhr.flush();
    });
    it('should cache template loaded through XHR', (done) {
      var firstEl;
      xhr.expect('/foo', 'xhr template');
      var template = new Template(url: '/foo');
      loader.load(template).then((el) {
        firstEl = el;
        return loader.load(template);
      }).then((el) {
        expect(el).toBe(firstEl);
        expect(el.content).toHaveText('xhr template');
        done();
      });
      xhr.flush();
    });
    it('should throw when no template is defined', () {
      var template = new Template(inline: null, url: null);
      expect(() {
        return loader.load(template);
      }).toThrowError(
          'Templates should have either their url or inline property set');
    });
    it('should return a rejected Promise when xhr loading fails', (done) {
      xhr.expect('/foo', null);
      var template = new Template(url: '/foo');
      PromiseWrapper.then(loader.load(template), (_) {
        throw 'Unexpected response';
      }, (error) {
        expect(error).toEqual('Failed to load /foo');
        done();
      });
      xhr.flush();
    });
  });
}
class SomeComponent {}
