library angular2.test.directives.non_bindable_spec_dart;

import 'package:angular2/test_lib.dart'
    show describe, xit, it, expect, beforeEach, ddescribe, iit, el;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/src/facade/collection.dart' show Map, MapWrapper;
import 'package:angular2/src/facade/lang.dart' show Type, isPresent;
import 'package:angular2/di.dart' show Injector;
import 'package:angular2/change_detection.dart'
    show Lexer, Parser, ChangeDetector, dynamicChangeDetection;
import 'package:angular2/src/core/compiler/compiler.dart'
    show Compiler, CompilerCache;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart'
    show DirectiveMetadataReader;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show NativeShadowDomStrategy;
import 'package:angular2/src/core/annotations/annotations.dart'
    show Decorator, Component;
import 'package:angular2/src/core/annotations/template.dart' show Template;
import 'package:angular2/src/core/compiler/template_loader.dart'
    show TemplateLoader;
import 'package:angular2/src/core/compiler/template_resolver.dart'
    show TemplateResolver;
import 'package:angular2/src/core/dom/element.dart' show NgElement;
import 'package:angular2/src/directives/non_bindable.dart' show NonBindable;

main() {
  describe('non-bindable', () {
    var view;
    var cd;
    var compiler;
    var component;
    var tplResolver;
    beforeEach(() {
      tplResolver = new FakeTemplateResolver();
      compiler = new Compiler(dynamicChangeDetection, new TemplateLoader(null),
          new DirectiveMetadataReader(), new Parser(new Lexer()),
          new CompilerCache(), new NativeShadowDomStrategy(), tplResolver);
    });
    createView(pv) {
      component = new TestComponent();
      view = pv.instantiate(null, null);
      view.hydrate(new Injector([]), null, component);
      cd = view.changeDetector;
    }
    compileWithTemplate(html) {
      var template =
          new Template(inline: html, directives: [NonBindable, TestDecorator]);
      tplResolver.setTemplate(TestComponent, template);
      return compiler.compile(TestComponent);
    }
    it('should not interpolate children', (done) {
      var template = '<div>{{text}}<span non-bindable>{{text}}</span></div>';
      compileWithTemplate(template).then((pv) {
        createView(pv);
        cd.detectChanges();
        expect(DOM.getText(view.nodes[0])).toEqual('foo{{text}}');
        done();
      });
    });
    it('should ignore directives on child nodes', (done) {
      var template =
          '<div non-bindable><span id=child test-dec>{{text}}</span></div>';
      compileWithTemplate(template).then((pv) {
        createView(pv);
        cd.detectChanges();
        var span = DOM.querySelector(view.nodes[0], '#child');
        expect(DOM.hasClass(span, 'compiled')).toBeFalsy();
        done();
      });
    });
    it('should trigger directives on the same node', (done) {
      var template =
          '<div><span id=child non-bindable test-dec>{{text}}</span></div>';
      compileWithTemplate(template).then((pv) {
        createView(pv);
        cd.detectChanges();
        var span = DOM.querySelector(view.nodes[0], '#child');
        expect(DOM.hasClass(span, 'compiled')).toBeTruthy();
        done();
      });
    });
  });
}
@Component(selector: 'test-cmp')
class TestComponent {
  String text;
  TestComponent() {
    this.text = 'foo';
  }
}
@Decorator(selector: '[test-dec]')
class TestDecorator {
  TestDecorator(NgElement el) {
    DOM.addClass(el.domElement, 'compiled');
  }
}
class FakeTemplateResolver extends TemplateResolver {
  Map _cmpTemplates;
  FakeTemplateResolver() : super() {
    this._cmpTemplates = MapWrapper.create();
  }
  setTemplate(Type component, Template template) {
    MapWrapper.set(this._cmpTemplates, component, template);
  }
  Template resolve(Type component) {
    var override = MapWrapper.get(this._cmpTemplates, component);
    if (isPresent(override)) {
      return override;
    }
    return super.resolve(component);
  }
}
